using cuqmbr.TravelGuide.Application.Common.Authorization;
using cuqmbr.TravelGuide.Application.Common.Persistence;
using cuqmbr.TravelGuide.Application.Common.Services;
using MediatR.Behaviors.Authorization;

namespace cuqmbr.TravelGuide.Application.TicketGroups.Queries.GetTicketGroup;

public class GetTicketGroupQueryAuthorizer :
    AbstractRequestAuthorizer<GetTicketGroupQuery>
{
    private readonly SessionUserService _sessionUserService;
    private readonly UnitOfWork _unitOfWork;

    public GetTicketGroupQueryAuthorizer(
        SessionUserService sessionUserService,
        UnitOfWork unitOfWork)
    {
        _sessionUserService = sessionUserService;
        _unitOfWork = unitOfWork;
    }

    public override void BuildPolicy(GetTicketGroupQuery request)
    {
        UseRequirement(new MustBeAuthenticatedRequirement
        {
            IsAuthenticated = _sessionUserService.IsAuthenticated
        });

        var ticketGroup = _unitOfWork.TicketGroupRepository
            .GetOneAsync(
                e => e.Guid == request.Guid, e => e.Account!,
                CancellationToken.None)
            .Result;

        UseRequirement(new MustBeObjectOwnerOrAdminRequirement
        {
            UserRoles = _sessionUserService.Roles,
            RequiredGuid = ticketGroup?.Account?.Guid,
            UserGuid = _sessionUserService.Guid
        });
    }
}
