using cuqmbr.TravelGuide.Application.Common.Services;
using cuqmbr.TravelGuide.Domain.Enums;
using FluentValidation;
using Microsoft.Extensions.Localization;

namespace cuqmbr.TravelGuide.Application.TicketGroups.Queries.GetTicketGroupsPage;

public class GetTicketGroupsPageQueryValidator : AbstractValidator<GetTicketGroupsPageQuery>
{
    public GetTicketGroupsPageQueryValidator(
        IStringLocalizer localizer,
        SessionCultureService cultureService)
    {
        RuleFor(v => v.PageNumber)
            .GreaterThanOrEqualTo(1)
            .WithMessage(
                String.Format(
                    cultureService.Culture,
                    localizer["FluentValidation.GreaterThanOrEqualTo"],
                    1));

        RuleFor(v => v.PageSize)
            .GreaterThanOrEqualTo(1)
            .WithMessage(
                String.Format(
                    cultureService.Culture,
                    localizer["FluentValidation.GreaterThanOrEqualTo"],
                    1))
            .LessThanOrEqualTo(50)
            .WithMessage(
                String.Format(
                    cultureService.Culture,
                    localizer["FluentValidation.LessThanOrEqualTo"],
                    50));

        RuleFor(v => v.Search)
            .MaximumLength(64)
            .WithMessage(
                String.Format(
                    cultureService.Culture,
                    localizer["FluentValidation.MaximumLength"],
                    64));

        When(v => v.PassangerSex != null, () =>
        {
            RuleForEach(v => v.PassangerSex)
                .Must((v, s) => Sex.Enumerations.ContainsValue(s))
                .WithMessage(
                    String.Format(
                        localizer["FluentValidation.MustBeInEnum"],
                        String.Join(
                            ", ",
                            Sex.Enumerations.Values.Select(e => e.Name))));
        });

        When(v => v.Statuses != null, () =>
        {
            RuleForEach(v => v.Statuses)
                .Must((v, s) => TicketStatus.Enumerations.ContainsValue(s))
                .WithMessage(
                    String.Format(
                        localizer["FluentValidation.MustBeInEnum"],
                        String.Join(
                            ", ",
                            TicketStatus.Enumerations.Values.Select(e => e.Name))));
        });

        When(v => v.VehicleTypes != null, () =>
        {
            RuleForEach(v => v.VehicleTypes)
                .Must((v, s) => VehicleType.Enumerations.ContainsValue(s))
                .WithMessage(
                    String.Format(
                        localizer["FluentValidation.MustBeInEnum"],
                        String.Join(
                            ", ",
                            VehicleType.Enumerations.Values.Select(e => e.Name))));
        });
    }
}
