using cuqmbr.TravelGuide.Application.Common.Mappings;
using cuqmbr.TravelGuide.Domain.Entities;

namespace cuqmbr.TravelGuide.Application.TicketGroups;

public sealed class TicketGroupVehicleEnrollmentDto : IMapFrom<Ticket>
{
    public DateTimeOffset DepartureTime { get; set; }

    public DateTimeOffset ArrivalTime { get; set; }

    public TimeSpan TravelTime { get; set; }

    public TimeSpan TimeMoving { get; set; }

    public TimeSpan TimeInStops { get; set; }

    public int NumberOfStops { get; set; }

    public string Currency { get; set; }

    public decimal Cost { get; set; }

    public Guid Uuid { get; set; }

    public short Order { get; set; }

    public TicketGroupCompanyDto Company { get; set; }

    public TicketGroupVehicleDto Vehicle { get; set; }

    public ICollection<TicketGroupAddressDto> Addresses { get; set; }

    public void Mapping(MappingProfile profile)
    {
        profile.CreateMap<Ticket, TicketGroupVehicleEnrollmentDto>()
            .ForMember(
                d => d.Uuid,
                opt => opt.MapFrom(s => s.VehicleEnrollment.Guid))
            .ForMember(
                d => d.DepartureTime,
                opt => opt.MapFrom(
                    (s, d) =>
                    {
                        return s.VehicleEnrollment
                            .GetDepartureTime(s.DepartureRouteAddressId);
                    }))
            .ForMember(
                d => d.ArrivalTime,
                opt => opt.MapFrom(
                    (s, d) =>
                    {
                        return s.VehicleEnrollment
                            .GetArrivalTime(s.ArrivalRouteAddressId);
                    }))
            .ForMember(
                d => d.TravelTime,
                opt => opt.MapFrom(
                    (s, d) =>
                    {
                        var departureTime = s.VehicleEnrollment
                            .GetDepartureTime(s.DepartureRouteAddressId);
                        var arrivalTime = s.VehicleEnrollment
                            .GetArrivalTime(s.ArrivalRouteAddressId);
                        return arrivalTime - departureTime;
                    }))
            .ForMember(
                d => d.TimeInStops,
                opt => opt.MapFrom(
                    (s, d) =>
                    {
                        return s.VehicleEnrollment.GetTimeInStops(
                            s.DepartureRouteAddressId, s.ArrivalRouteAddressId);
                    }))
            .ForMember(
                d => d.NumberOfStops,
                opt => opt.MapFrom(
                    (s, d) =>
                    {
                        return s.VehicleEnrollment.GetNumberOfStops(
                            s.DepartureRouteAddressId, s.ArrivalRouteAddressId);
                    }))
            .ForMember(
                d => d.Currency,
                opt => opt.MapFrom(s => s.Currency))
            .ForMember(
                d => d.Cost,
                opt => opt.MapFrom(s => s.Currency.Round(s.Cost)))
            .ForMember(
                d => d.Company,
                opt => opt.MapFrom(s => s.VehicleEnrollment.Vehicle.Company))
            .ForMember(
                d => d.Vehicle,
                opt => opt.MapFrom(s => s.VehicleEnrollment.Vehicle))
            .ForMember(
                d => d.Addresses,
                opt => opt.MapFrom(s => s.VehicleEnrollment.RouteAddressDetails));
    }
}
