using cuqmbr.TravelGuide.Application.Common.Authorization;
using cuqmbr.TravelGuide.Application.Common.Persistence;
using cuqmbr.TravelGuide.Application.Common.Services;
using MediatR.Behaviors.Authorization;

namespace cuqmbr.TravelGuide.Application.Trains.Queries.GetTrain;

public class GetTrainQueryAuthorizer :
    AbstractRequestAuthorizer<GetTrainQuery>
{
    private readonly SessionUserService _sessionUserService;
    private readonly UnitOfWork _unitOfWork;

    public GetTrainQueryAuthorizer(
        SessionUserService sessionUserService,
        UnitOfWork unitOfWork)
    {
        _sessionUserService = sessionUserService;
        _unitOfWork = unitOfWork;
    }

    public override void BuildPolicy(GetTrainQuery request)
    {
        UseRequirement(new MustBeAuthenticatedRequirement
        {
            IsAuthenticated = _sessionUserService.IsAuthenticated
        });

        var vehicel = _unitOfWork.VehicleRepository
            .GetOneAsync(
                e => e.Guid == request.Guid, e => e.Company.Account,
                CancellationToken.None)
            .Result;

        UseRequirement(new MustBeObjectOwnerOrAdminRequirement
        {
            UserRoles = _sessionUserService.Roles,
            RequiredGuid = vehicel?.Company.Account.Guid,
            UserGuid = _sessionUserService.Guid
        });
    }
}
