using cuqmbr.TravelGuide.Application.Common.Authorization;
using cuqmbr.TravelGuide.Application.Common.Persistence;
using cuqmbr.TravelGuide.Application.Common.Services;
using MediatR.Behaviors.Authorization;

namespace cuqmbr.TravelGuide.Application.VehicleEnrollments
    .Queries.GetVehicleEnrollment;

public class GetVehicleEnrollmentQueryAuthorizer :
    AbstractRequestAuthorizer<GetVehicleEnrollmentQuery>
{
    private readonly SessionUserService _sessionUserService;
    private readonly UnitOfWork _unitOfWork;

    public GetVehicleEnrollmentQueryAuthorizer(
        SessionUserService sessionUserService,
        UnitOfWork unitOfWork)
    {
        _sessionUserService = sessionUserService;
        _unitOfWork = unitOfWork;
    }

    public override void BuildPolicy(GetVehicleEnrollmentQuery request)
    {
        UseRequirement(new MustBeAuthenticatedRequirement
        {
            IsAuthenticated = _sessionUserService.IsAuthenticated
        });

        var vehicelEnrollment = _unitOfWork.VehicleEnrollmentRepository
            .GetOneAsync(
                e => e.Guid == request.Guid, e => e.Vehicle.Company.Account,
                CancellationToken.None)
            .Result;

        UseRequirement(new MustBeObjectOwnerOrAdminRequirement
        {
            UserRoles = _sessionUserService.Roles,
            RequiredGuid = vehicelEnrollment?.Vehicle.Company.Account.Guid,
            UserGuid = _sessionUserService.Guid
        });
    }
}
